/********************************************************/
/*	SAMPLE PROGRAM 3 FOR MSA0654-MEAUST BOARD	*/
/*	FILENAME: sample3.c				*/
/*	WRITTEN BY: Alan Suyko  28/7/99			*/
/*	DESCRIPTION:  					*/
/*	-- Uses port P0 and P1 and Timer A0 to display 	*/
/*	count up sequence on LED1 and LED2 at 1 sec 	*/
/*	update interval.  				*/
/*	-- Uses Timer A1 to multiplex refresh of LED1	*/
/*	and LED2.					*/
/*							*/
/*	Copyright 1999 by MITSUBISHI ELECTRIC AUSTRALIA	*/
/*	All rights reserved. See Terms and Conditions	*/
/*	document regarding the use of this sample	*/
/*	program.					*/
/********************************************************/ 	

#include "sfr62.h"
#include "leddata.h"
void main(void);
void initport(void);
void initTimerA0(void);
void initTimerA1(void);
void startTimerA0(void);
void startTimerA1(void);
void setTimerA0int(void);
void setTimerA1int(void);
void TimerA0int(void);
void TimerA1int(void);

#pragma INTERRUPT TimerA0int
#pragma INTERRUPT TimerA1int

unsigned char num_sec = 0;

void main(void)
{
	initport();
	initTimerA0();
	initTimerA1();
	setTimerA0int();
	setTimerA1int();
	startTimerA0();
	startTimerA1();
	while(1);	// endless loop
}

void initport(void)
{
	pd0 = 0xFF;	// output mode 
	pd1 = 0xFF;	// output mode 
	pu00 = 0;	// no pull up for P0_0 to P0_3 
	pu01 = 0;	// no pull up for P0_4 to P0_7 
	pu02 = 0;	// no pull up for P1_0 to P1_3 
	p1 = 0xFD;	// enable LED2 and disable LED1
	p0 = 0xFF;	// turn off all segments
}


void initTimerA0(void)
{
    	ta0mr = 0x80;       // XX0X XX00 
                            // |||| |||+- must always be 0 in timer mode
                            // |||| ||+-- must always be 0 in timer mode
                            // |||| |+--- 0: pulse is not output at pin TA0out
                            // |||| |     1: pulse is output at pin TA0out
                            // |||| |        TA0out is automatically  output
                            // |||| +---- 0: gate function: timer counts only 
                            // ||||          when TA0in is held "L"
                            // ||||       1: gate function: timer counts only
                            // ||||          when TA0in is held "H"
                            // |||+------ 0: gate function not available
                            // |||        1: gate function available
                            // ||+------- must always be 0 in timer mode
                            // |+-------- count source select bits:
                            // +--------- count source select bits:
                            //            00:  f1
                            //            01:  f8
                            //            10:  f32
                            //            11:  fc32

    	ta0 = 0xC350;	    // Set up Timer A0 Reload Register for 0.1 sec interval interrupt
}

void initTimerA1(void)
{       		
    	ta1mr = 0x80;       // XX0X XX00 
                            // |||| |||+- must always be 0 in timer mode
                            // |||| ||+-- must always be 0 in timer mode
                            // |||| |+--- 0: pulse is not output at pin TA1out
                            // |||| |     1: pulse is output at pin TA1out
                            // |||| |        TA0out is automatically  output
                            // |||| +---- 0: gate function: timer counts only 
                            // ||||          when TA0in is held "L"
                            // ||||       1: gate function: timer counts only
                            // ||||          when TA0in is held "H"
                            // |||+------ 0: gate function not available
                            // |||        1: gate function available
                            // ||+------- must always be 0 in timer mode
                            // |+-------- count source select bits:
                            // +--------- count source select bits:
                            //            00:  f1
                            //            01:  f8
                            //            10:  f32
                            //            11:  fc32

    	ta1 = 0x1388;	    // Set up Timer A1 Reload Register for 10 msec interval
			    // for a LED refresh rate of 100Hz
}

void startTimerA0(void)
{
    tabsr |= 0x01;          //	1: start timer A0 (count flag)
}

void startTimerA1(void)
{
    tabsr |= 0x02;          //	1: start timer A1 (count flag)
}

void setTimerA0int(void)
{
    ta0ic = 0x01;           // ---- XXXX
                            //      ||||
                            //      |||+-- Interupt priority level select bit
                            //      ||+--- Interupt priority level select bit
                            //      |+---- Interupt priority level select bit
                            //      |      000: Level 0 (interrupt disabled)
                            //      |      001: Level 1
                            //      |      010: Level 2
                            //      |      011: Level 3
                            //      |      100: Level 4
                            //      |      101: Level 5
                            //      |      110: Level 6
                            //      |      111: Level 7
                            //      +----- Interupt request bit 
                            //             0: Interrupt not requested
                            //             1: Interrupt requested
}

void setTimerA1int(void)
{
    ta1ic = 0x01;           // ---- XXXX
                            //      ||||
                            //      |||+-- Interupt priority level select bit
                            //      ||+--- Interupt priority level select bit
                            //      |+---- Interupt priority level select bit
                            //      |      000: Level 0 (interrupt disabled)
                            //      |      001: Level 1
                            //      |      010: Level 2
                            //      |      011: Level 3
                            //      |      100: Level 4
                            //      |      101: Level 5
                            //      |      110: Level 6
                            //      |      111: Level 7
                            //      +----- Interupt request bit 
                            //             0: Interrupt not requested
                            //             1: Interrupt requested
}

void TimerA0int(void)		// interrupt function
{	
	static unsigned char one_hundred_msec_count = 0;

	asm("fset I");		// enable interrupts

	one_hundred_msec_count++;
	if (one_hundred_msec_count == 10)	// 1 sec has elapsed if true
		{
		one_hundred_msec_count = 0;
		num_sec++;
		if (num_sec == 100)
			num_sec = 0;
		}
}

void TimerA1int(void)		// interrupt function
{	
	static unsigned char led_select = 0;
	unsigned char offset;

	asm("fset I");		// enable interrupts
	if (led_select == 0)
		{
		led_select = 1;
		p1 = 0xFD;		// enable LED2 and disable LED1 
		offset = num_sec % 10;	
		p0 = leddigit[offset];	// display remainder
		}
	else
		{
		led_select = 0;
		p1 = 0xFE;		// disable LED2 and enable LED1 
		offset = num_sec / 10;	
		p0 = leddigit[offset];	// display quotient
		}
}


